<?php

/*
Written by Chris Jean for iThemes.com
Version 1.4.2

Version History
	1.1.0 - 2011-02-22 - Chris Jean
		Added functions:
			show_inline_status_message
			show_inline_error_message
			sort_array
			get_array_value
			add_array_value
	1.2.0 - 2011-06-27 - Chris Jean
		Added functions:
			add_tooltip_scripts
			add_tooltip_styles
			get_tooltip
			add_tooltip
	1.3.0 - 2011-08-04 - Chris Jean
		Added functions:
			fix_url
			get_random_string
	1.4.0 - 2011-08-23 - Chris Jean
		Added debug functions:
			backtrace
			inspect
			pad
			print_r
	1.4.1 - 2011-10-06 - Chris Jean
		Simplified verify_file code
	1.4.2 - 2011-10-20 - Chris Jean
		Removed instances of TEMPLATEPATH and STYLESHEETPATH
*/


if ( ! class_exists( 'ITUtility' ) ) {
	it_classes_load( 'it-error.php' );
	
	class ITUtility {
		function require_file_once( $file, $class = false ) {
			$file = ITUtility::verify_file( $file, 'require_once', true );
			@require_once( $file );
			ITUtility::check_class( $class, true );
		}
		
		function require_file( $file, $class = false ) {
			$file = ITUtility::verify_file( $file, 'require', true );
			@require( $file );
			ITUtility::check_class( $class, true );
		}
		
		function include_file_once( $file, $class = false ) {
			$file = ITUtility::verify_file( $file, 'include_once', false );
			@include_once( $file );
			ITUtility::check_class( $class, false );
		}
		
		function include_file( $file, $class = false ) {
			$file = ITUtility::verify_file( $file, 'include', false );
			@include( $file );
			ITUtility::check_class( $class, false );
		}
		
		function verify_file( $file, $type, $required = true ) {
			if ( file_exists( get_stylesheet_directory() . "/$file" ) )
				return get_stylesheet_directory() . "/$file";
			else if ( file_exists( get_template_directory() . "/$file" ) )
				return get_template_directory() . "/$file";
			else if ( file_exists( ABSPATH . "/$file" ) )
				return ABSPATH . "/$file";
			else if ( file_exists( dirname( __FILE__ ) . "/$file" ) )
				return dirname( __FILE__ ) . "/$file";
			else if ( ! file_exists( $file ) ) {
				if ( $required )
					ITError::fatal( "missing_file:$type:$file", 'A file necessary for the theme to function is missing or unable to be read by the web server.' );
				else {
					ITError::warn( "missing_file:$type:$file", 'A theme file is missing or unable to be read by the web server.' );
					return false;
				}
			}
			
			return $file;
		}
		
		function check_class( $class, $required = true ) {
			if ( false !== $class ) {
				if ( ! class_exists( $class ) ) {
					if ( $required )
						ITError::fatal( "missing_class:$class", 'A necessary core component of the theme is missing.' );
					else
						ITError::warn( "missing_class:$class", 'A core component of the theme is missing.' );
				}
			}
		}
		
		function merge_defaults( $values, $defaults, $force = false ) {
			if ( ! ITUtility::is_associative_array( $defaults ) ) {
				if ( ! isset( $values ) )
					return $defaults;
				
				if ( false === $force )
					return $values;
				
				if ( isset( $values ) || is_array( $values ) )
					return $values;
				return $defaults;
			}
			
			foreach ( (array) $defaults as $key => $val ) {
				if ( ! isset( $values[$key] ) )
					$values[$key] = null;
				
				$values[$key] = ITUtility::merge_defaults( $values[$key], $val, $force );
			}
			
			return $values;
		}
		
		function is_associative_array( &$array ) {
			if ( ! is_array( $array ) || empty( $array ) )
				return false;
			
			$next = 0;
			
			foreach ( $array as $k => $v )
				if ( $k !== $next++ )
					return true;
			
			return false;
		}
		
		function show_status_message( $message ) {
			echo "<div class=\"updated fade\"><p><strong>$message</strong></p></div>\n";
		}
		
		function show_error_message( $message ) {
			if ( is_wp_error( $message ) )
				$message = $message->get_error_message();
			
			if ( ! is_string( $message ) )
				return;
			
			echo "<div class=\"error\"><p><strong>$message</strong></p></div>\n";
		}
		
		function show_inline_status_message( $message ) {
			echo "<div class=\"updated fade inline\"><p><strong>$message</strong></p></div>\n";
		}
		
		function show_inline_error_message( $message ) {
			if ( is_wp_error( $message ) )
				$message = $message->get_error_message();
			
			if ( ! is_string( $message ) )
				return;
			
			echo "<div class=\"error inline\"><p><strong>$message</strong></p></div>\n";
		}
		
		function verify_class( $var, $class ) {
			if ( isset( $var ) && is_object( $var ) && ( strtolower( $class ) === strtolower( get_class( $var ) ) ) )
				return true;
			return false;
		}
		
		function get_open_tag( $tag_name, $attributes = array() ) {
			$tag = "<$tag_name";
			
			foreach ( (array) $attributes as $attribute => $values ) {
				$attr_value = '';
				foreach ( (array) $values as $value ) {
					if ( ! empty( $attr_value ) )
						$attr_value .= ' ';
					$attr_value .= str_replace( '"', '&quot;', $value );
				}
				$tag .= " $attribute=\"$attr_value\"";
			}
			
			$tag .= '>';
			
			return $tag;
		}
		
		function print_open_tag( $tag_name, $attributes ) {
			echo ITUtility::get_open_tag( $tag_name, $attributes ) . "\n";
		}
		
		function cleanup_request_vars() {
			$_REQUEST = ITUtility::strip_slashes( $_REQUEST );
			$_POST = ITUtility::strip_slashes( $_POST );
			$_GET = ITUtility::strip_slashes( $_GET );
		}
		
		function strip_slashes( $var ) {
			if ( is_array( $var ) ) {
				foreach ( (array) $var as $index => $val )
					$var[$index] = ITUtility::strip_slashes( $val );
			}
			else
				$var = stripslashes( $var );
			
			return $var;
		}
		
		function sort_array( $array, $index, $args = array() ) {
			it_classes_load( 'it-array-sort.php' );
			
			$sorter = new ITArraySort( $array, $index, $args );
			
			return $sorter->get_sorted_array();
		}
		
		// Deprecated
		function sort_array_by_index( $array, $index ) {
			if ( ! is_array( $array ) )
				ITError::fatal( 'invalid_var:parameter:array', 'Invalid data was passed to ITUtility::sort_array_by_index. This indicates a code bug.' );
			
			$new_array = array();
			$indexes = array();
			
			foreach ( (array) $array as $sub_index => $sub_array )
				$indexes[$sub_index] = $sub_array[$index];
			
			asort( $indexes );
			
			foreach ( (array) $indexes as $sub_index => $sub_value )
				$new_array[] = $array[$sub_index];
			
			return $new_array;
		}
		
		function get_array_value( $array, $index ) {
			if ( is_string( $index ) ) {
				if ( false === strpos( $index, '[' ) )
					$index = array( $index );
				else {
					$index = rtrim( $index, '[]' );
					$index = preg_split( '/[\[\]]+/', $index );
				}
			}
			
			while ( count( $index ) > 1 ) {
				if ( isset( $array[$index[0]] ) ) {
					$array = $array[$index[0]];
					array_shift( $index );
				}
				else
					return null;
			}
			
			if ( isset( $array[$index[0]] ) )
				return $array[$index[0]];
			
			return null;
		}
		
		function add_array_value( &$array, $index, $val ) {
			if ( is_string( $index ) ) {
				if ( false === strpos( $index, '[' ) )
					$index = array( $index );
				else {
					$index = rtrim( $index, '[]' );
					$index = preg_split( '/[\[\]]+/', $index );
				}
			}
			
			$cur_array =& $array;
			
			while ( count( $index ) > 1 ) {
				if ( ! isset( $cur_array[$index[0]] ) || ! is_array( $cur_array[$index[0]] ) )
					$cur_array[$index[0]] = array();
				
				$cur_array =& $cur_array[$index[0]];
				array_shift( $index );
			}
			
			$cur_array[$index[0]] = $val;
		}
		
		function print_js_vars( $options = array() ) {
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
			<?php foreach ( (array) $options as $var => $val ) : ?>
				<?php $val = str_replace( '"', '\\"', $val ); ?>
				var <?php echo $var; ?> = "<?php echo $val; ?>";
			<?php endforeach; ?>
		/* ]]> */
	</script>
<?php
			
		}
		
		function print_js_script( $script ) {
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
			<?php echo $script; ?>
		/* ]]> */
	</script>
<?php
			
		}
		
		function add_tooltip_scripts() {
			global $it_utility_cached_url_base;
			
			if ( empty( $it_utility_cached_url_base ) )
				$it_utility_cached_url_base = ITFileUtility::get_url_from_file( dirname( __FILE__ ) );
			
			wp_enqueue_script( 'jquery-tooltip', $it_utility_cached_url_base . '/js/jquery.tooltip.js', array( 'jquery' ) );
			wp_enqueue_script( 'it-tooltip', $it_utility_cached_url_base . '/js/it-tooltip.js', array( 'jquery-tooltip' ) );
		}
		
		function add_tooltip_styles() {
			global $it_utility_cached_url_base;
			
			if ( empty( $it_utility_cached_url_base ) )
				$it_utility_cached_url_base = ITFileUtility::get_url_from_file( dirname( __FILE__ ) );
			
			wp_enqueue_style( 'it-tooltip', $it_utility_cached_url_base . '/css/it-tooltip.css' );
		}
		
		function get_tooltip( $message, $title = '', $class = '', $alt = '(?)' ) {
			global $it_utility_cached_url_base;
			
			if ( empty( $it_utility_cached_url_base ) )
				$it_utility_cached_url_base = ITFileUtility::get_url_from_file( dirname( __FILE__ ) );
			
			$message = esc_attr( $message );
			$title = esc_attr( $title );
			
			if ( empty( $class ) )
				$class = 'it-tooltip';
			else
				$class = "it-tooltip-$class";
			
			$tip = "<a class='$class' title='$title |:|~| $message'><img src='" . $it_utility_cached_url_base . "/images/it-tooltip.png' alt='(?)' /></a>";
			
			return $tip;
		}
		
		function add_tooltip( $message, $title = '', $class = '', $alt = '(?)' ) {
			echo ITUtility::get_tooltip( $message, $title, $class, $alt );
		}
		
		/* Automatically changes http protocols to https when is_ssl is true */
		function fix_url( $url ) {
			if ( ! is_ssl() )
				return $url;
			
			return preg_replace( '|^http://|', 'https://', $url );
		}
		
		function get_random_string( $length = 10, $use_sets = array( 'lower', 'upper', 'num' ) ) {
			$sets = array(
				'lower'    => 'abcdefghijklmnopqrstuvwxyz',
				'upper'    => 'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
				'num'      => '0123456789',
				'basic'    => '-_,.',
				'extended' => '`~!@#$%^&*()=+[]{};:\'"<>/?\|',
			);
			
			
			if ( is_array( $length ) ) {
				if ( ! isset( $length[0] ) || ! isset( $length[1] ) )
					return '';
				
				$lower = intval( $length[0] );
				$upper = intval( $length[1] );
				
				$length = rand( $lower, $upper );
			}
			else
				$length = intval( $length );
			
			if ( $length < 1 )
				$length = 1;
			
			
			$source_string = '';
			
			if ( is_string( $use_sets ) )
				$source_string = $use_sets;
			else if ( is_array( $use_sets ) ) {
				foreach ( $use_sets as $set ) {
					if ( is_string( $set ) && isset( $sets[$set] ) )
						$source_string .= $sets[$set];
				}
			}
			else if ( true === $use_sets ) {
				foreach ( $sets as $chars )
					$source_string .= $chars;
			}
			
			if ( empty( $source_string ) )
				return false;
			
			
			$string = '';
			
			while ( strlen( $string ) < $length )
				$string .= substr( $source_string, rand( 0, strlen( $source_string ) - 1 ), 1 );
			
			
			return $string;
		}
		
		function pad( $depth, $pad = '    ' ) {
			$retval = '';
			
			for ( $x = 0; $x <= $depth; $x++ )
				$retval .= $pad;
			
			return $retval;
		}
		
		
		/* Debug Functions */
		
		function print_r( $data, $description = '', $expand_objects = true, $max_depth = -1 ) {
			if ( ! empty( $description ) )
				$description .= "\n";
			
			echo "<pre style='color:black;background:white;padding:15px;font-family:\"Courier New\",Courier,monospace;font-size:12px;'>$description" . ITUtility::inspect( $data, $expand_objects, $max_depth, false ) . "</pre>\n";
		}
		
		function backtrace( $description = false, $expand_objects = false, $max_depth = 3 ) {
			if ( ! is_string( $description ) || empty( $description ) )
				$description = 'Backtrace';
			
			$backtrace = debug_backtrace();
			unset( $backtrace[0] );
			
			ITUtility::print_r( $backtrace, $description, $expand_objects, $max_depth + 1 );
		}
		
		function inspect( $data, $expand_objects = false, $max_depth = 2, $echo = true ) {
			// Create a deep copy so that variables aren't needlessly manipulated.
			$data = unserialize( serialize( $data ) );
			
			if ( $max_depth < 1 )
				$max_depth = 10;
			
			$retval = ITUtility::_inspect_dive( $data, $expand_objects, $max_depth );
			
			if ( $echo )
				echo $retval;
			return $retval;
		}
		
		function _inspect_dive( $data, $expand_objects, $max_depth, $depth = 0, $show_array_header = true ) {
			$pad = ITUtility::pad( $depth, '    ' );
			
			if ( is_string( $data ) ) {
				if ( empty( $data ) )
					return "<strong>[empty string]</strong>";
				else
					return htmlspecialchars( $data );
			}
			
			if ( is_bool( $data ) )
				return ( $data ) ? '<strong>true</strong>' : '<strong>false</strong>';
			
			if ( is_null( $data ) )
				return '<strong>null</strong>';
			
			if ( is_object( $data ) ) {
				$class_name = get_class( $data );
				$retval = "<strong>Object</strong> $class_name";
				
				if ( ! $expand_objects || ( $depth == $max_depth ) )
					return $retval;
				
				$vars = get_object_vars( $data );
				
				if ( empty( $vars ) )
					$vars = '';
				else
					$vars = ITUtility::_inspect_dive( $vars, $expand_objects, $max_depth, $depth, false );
				
				$retval .= "$vars";
				
				return $retval;
			}
			
			if ( is_array( $data ) ) {
				$retval = ( $show_array_header ) ? '<strong>Array</strong>' : '';
				
				if ( empty( $data ) )
					return "$retval()";
				if ( $depth == $max_depth )
					return "$retval( ... )";
				
				$max = 0;
				
				foreach ( array_keys( $data ) as $index ) {
					if ( strlen( $index ) > $max )
						$max = strlen( $index );
				}
				
				foreach ( $data as $index => $val )
					$retval .= "\n$pad" . sprintf( "%-{$max}s", $index ) . "  <strong>=&gt;</strong> " . ITUtility::_inspect_dive( $val, $expand_objects, $max_depth, $depth + 1 );
				
				return $retval;
			}
			
			return '<strong>[' . gettype( $data ) . ']</strong> ' . $data;
		}
	}
}
